@@ -6,7 +6,7 @@
 # http://www.apache.org/licenses/LICENSE-2.0OA
 #
 # Authors:
-# - Wen Guan, <wen.guan@cern.ch>, 2020 - 2021
+# - Wen Guan, <wen.guan@cern.ch>, 2020 - 2022
 # - Sergey Padolski, <spadolski@bnl.gov>, 2020
 
 
@@ -48,7 +48,8 @@ class DomaPanDAWork(Work):
                  num_retries=5,
                  task_log=None,
                  task_cloud=None,
-                 task_rss=0):
+                 task_site=None,
+                 task_rss=1000):
 
         super(DomaPanDAWork, self).__init__(executable=executable, arguments=arguments,
                                             parameters=parameters, setup=setup, work_type=TransformType.Processing,
@@ -66,12 +67,16 @@ class DomaPanDAWork(Work):
         self.panda_url = None
         self.panda_url_ssl = None
         self.panda_monitor = None
+        self.panda_auth = None
+        self.panda_auth_vo = None
+        self.panda_config_root = None
 
         self.dependency_map = dependency_map
         self.dependency_map_deleted = []
         # self.logger.setLevel(logging.DEBUG)
 
         self.task_name = task_name
+        self.real_task_name = None
         self.set_work_name(task_name)
         self.queue = task_queue
         self.dep_tasks_id_names_map = {}
@@ -86,11 +91,14 @@ class DomaPanDAWork(Work):
 
         self.encode_command_line = encode_command_line
         self.task_cloud = task_cloud
+        self.task_site = task_site
         self.task_rss = task_rss
 
         self.retry_number = 0
         self.num_retries = num_retries
 
+        self.poll_panda_jobs_chunk_size = 2000
+
         self.load_panda_urls()
 
     def my_condition(self):
@@ -119,6 +127,9 @@ class DomaPanDAWork(Work):
         self.panda_url = None
         self.panda_url_ssl = None
         self.panda_monitor = None
+        self.panda_auth = None
+        self.panda_auth_vo = None
+        self.panda_config_root = None
 
         if panda_config.has_section('panda'):
             if panda_config.has_option('panda', 'panda_monitor_url'):
@@ -133,6 +144,15 @@ class DomaPanDAWork(Work):
                 self.panda_url_ssl = panda_config.get('panda', 'panda_url_ssl')
                 os.environ['PANDA_URL_SSL'] = self.panda_url_ssl
                 # self.logger.debug("Panda url ssl: %s" % str(self.panda_url_ssl))
+            if panda_config.has_option('panda', 'panda_auth'):
+                self.panda_auth = panda_config.get('panda', 'panda_auth')
+                os.environ['PANDA_AUTH'] = self.panda_auth
+            if panda_config.has_option('panda', 'panda_auth_vo'):
+                self.panda_auth_vo = panda_config.get('panda', 'panda_auth_vo')
+                os.environ['PANDA_AUTH_VO'] = self.panda_auth_vo
+            if panda_config.has_option('panda', 'panda_config_root'):
+                self.panda_config_root = panda_config.get('panda', 'panda_config_root')
+                os.environ['PANDA_CONFIG_ROOT'] = self.panda_config_root
 
         if not self.panda_monitor and 'PANDA_MONITOR_URL' in os.environ and os.environ['PANDA_MONITOR_URL']:
             self.panda_monitor = os.environ['PANDA_MONITOR_URL']
@@ -143,6 +163,12 @@ class DomaPanDAWork(Work):
         if not self.panda_url_ssl and 'PANDA_URL_SSL' in os.environ and os.environ['PANDA_URL_SSL']:
             self.panda_url_ssl = os.environ['PANDA_URL_SSL']
             # self.logger.debug("Panda url ssl: %s" % str(self.panda_url_ssl))
+        if not self.panda_auth and 'PANDA_AUTH' in os.environ and os.environ['PANDA_AUTH']:
+            self.panda_auth = os.environ['PANDA_AUTH']
+        if not self.panda_auth_vo and 'PANDA_AUTH_VO' in os.environ and os.environ['PANDA_AUTH_VO']:
+            self.panda_auth_vo = os.environ['PANDA_AUTH_VO']
+        if not self.panda_config_root and 'PANDA_CONFIG_ROOT' in os.environ and os.environ['PANDA_CONFIG_ROOT']:
+            self.panda_config_root = os.environ['PANDA_CONFIG_ROOT']
 
     def set_agent_attributes(self, attrs, req_attributes=None):
         if 'life_time' not in attrs[self.class_name] or int(attrs[self.class_name]['life_time']) <= 0:
@@ -150,6 +176,8 @@ class DomaPanDAWork(Work):
         super(DomaPanDAWork, self).set_agent_attributes(attrs)
         if 'num_retries' in self.agent_attributes and self.agent_attributes['num_retries']:
             self.num_retries = int(self.agent_attributes['num_retries'])
+        if 'poll_panda_jobs_chunk_size' in self.agent_attributes and self.agent_attributes['poll_panda_jobs_chunk_size']:
+            self.poll_panda_jobs_chunk_size = int(self.agent_attributes['poll_panda_jobs_chunk_size'])
 
     def depend_on(self, work):
         for job in self.dependency_map:
@@ -161,6 +189,17 @@ class DomaPanDAWork(Work):
                     return True
         return False
 
+    def get_ancestry_works(self):
+        tasks = set([])
+        for job in self.dependency_map:
+            inputs_dependency = job["dependencies"]
+
+            for input_d in inputs_dependency:
+                task_name = input_d['task']
+                if task_name not in tasks:
+                    tasks.add(task_name)
+        return list(tasks)
+
     def poll_external_collection(self, coll):
         try:
             if coll.status in [CollectionStatus.Closed]:
@@ -366,7 +405,7 @@ class DomaPanDAWork(Work):
         task_param_map['noInput'] = True
         task_param_map['pfnList'] = in_files
         task_param_map['taskName'] = self.task_name
-        task_param_map['userName'] = 'iDDS'
+        task_param_map['userName'] = self.username if self.username else 'iDDS'
         task_param_map['taskPriority'] = 900
         task_param_map['architecture'] = ''
         task_param_map['transUses'] = ''
@@ -384,6 +423,7 @@ class DomaPanDAWork(Work):
         task_param_map['coreCount'] = self.core_count
         task_param_map['skipScout'] = True
         task_param_map['cloud'] = self.task_cloud
+        task_param_map['PandaSite'] = self.task_site
         if self.task_rss and self.task_rss > 0:
             task_param_map['ramCount'] = self.task_rss
             task_param_map['ramUnit'] = 'MB'
@@ -413,11 +453,11 @@ class DomaPanDAWork(Work):
 
     def submit_panda_task(self, processing):
         try:
-            from pandatools import Client
+            from pandaclient import Client
 
             proc = processing['processing_metadata']['processing']
             task_param = proc.processing_metadata['task_param']
-            return_code = Client.insertTaskParams(task_param, verbose=True)
+            return_code = Client.insertTaskParams(task_param, verbose=False)
             if return_code[0] == 0:
                 return return_code[1][1]
             else:
@@ -444,8 +484,20 @@ class DomaPanDAWork(Work):
             if task_id:
                 proc.submitted_at = datetime.datetime.utcnow()
 
+    def resubmit_processing(self, processing):
+        proc = processing['processing_metadata']['processing']
+        proc.workload_id = None
+        task_param = proc.processing_metadata['task_param']
+        if self.retry_number > 0:
+            proc.task_name = self.task_name + "_" + str(self.retry_number)
+            task_param['taskName'] = proc.task_name
+        task_id = self.submit_panda_task(processing)
+        proc.workload_id = task_id
+        if task_id:
+            proc.submitted_at = datetime.datetime.utcnow()
+
     def get_panda_task_id(self, processing):
-        from pandatools import Client
+        from pandaclient import Client
 
         start_time = datetime.datetime.utcnow() - datetime.timedelta(hours=10)
         start_time = start_time.strftime('%Y-%m-%d %H:%M:%S')
@@ -458,7 +510,10 @@ class DomaPanDAWork(Work):
         task_id = None
         for req_id in results:
             task_name = results[req_id]['taskName']
-            if proc.workload_id is None and task_name == self.task_name:
+            local_task_name = proc.task_name
+            if not local_task_name:
+                local_task_name = self.task_name
+            if proc.workload_id is None and task_name == local_task_name:
                 task_id = results[req_id]['jediTaskID']
                 # processing['processing_metadata']['task_id'] = task_id
                 # processing['processing_metadata']['workload_id'] = task_id
@@ -470,7 +525,7 @@ class DomaPanDAWork(Work):
 
     def poll_panda_task_status(self, processing):
         if 'processing' in processing['processing_metadata']:
-            from pandatools import Client
+            from pandaclient import Client
 
             proc = processing['processing_metadata']['processing']
             status, task_status = Client.getTaskStatus(proc.workload_id)
@@ -492,9 +547,11 @@ class DomaPanDAWork(Work):
         elif task_status in ['finished', 'paused']:
             # finished, finishing, waiting it to be done
             processing_status = ProcessingStatus.SubFinished
-        elif task_status in ['failed', 'aborted', 'broken', 'exhausted']:
+        elif task_status in ['failed', 'aborted', 'exhausted']:
             # aborting, tobroken
             processing_status = ProcessingStatus.Failed
+        elif task_status in ['broken']:
+            processing_status = ProcessingStatus.Broken
         else:
             # finished, finishing, aborting, topreprocess, preprocessing, tobroken
             # toretry, toincexec, rerefine, paused, throttled, passed
@@ -622,16 +679,37 @@ class DomaPanDAWork(Work):
             update_contents.append(content)
         return update_contents
 
+    def get_panda_job_status(self, jobids):
+        self.logger.debug("get_panda_job_status, jobids[:10]: %s" % str(jobids[:10]))
+        from pandaclient import Client
+        ret = Client.getJobStatus(jobids, verbose=0)
+        if ret[0] == 0:
+            left_jobids = []
+            ret_jobs = []
+            jobs_list = ret[1]
+            for jobid, jobinfo in zip(jobids, jobs_list):
+                if jobinfo is None:
+                    left_jobids.append(jobid)
+                else:
+                    ret_jobs.append(jobinfo)
+            if left_jobids:
+                ret1 = Client.getFullJobStatus(ids=left_jobids, verbose=False)
+                if ret1[0] == 0:
+                    left_jobs_list = ret1[1]
+                ret_jobs = ret_jobs + left_jobs_list
+            return ret_jobs
+        return []
+
     def map_panda_ids(self, unregistered_job_ids, input_output_maps):
-        self.logger.debug("map_panda_ids, unregistered_job_ids: %s" % str(unregistered_job_ids))
-        from pandatools import Client
+        self.logger.debug("map_panda_ids, unregistered_job_ids[:10]: %s" % str(unregistered_job_ids[:10]))
 
         # updated_map_ids = []
         full_update_contents = []
         chunksize = 2000
         chunks = [unregistered_job_ids[i:i + chunksize] for i in range(0, len(unregistered_job_ids), chunksize)]
         for chunk in chunks:
-            jobs_list = Client.getJobStatus(chunk, verbose=0)[1]
+            # jobs_list = Client.getJobStatus(chunk, verbose=0)[1]
+            jobs_list = self.get_panda_job_status(chunk)
             for job_info in jobs_list:
                 if job_info and job_info.Files and len(job_info.Files) > 0:
                     for job_file in job_info.Files:
@@ -651,14 +729,14 @@ class DomaPanDAWork(Work):
         return full_update_contents
 
     def get_status_changed_contents(self, unterminated_job_ids, input_output_maps, panda_id_to_map_ids):
-        self.logger.debug("get_status_changed_contents, unterminated_job_ids: %s" % str(unterminated_job_ids))
-        from pandatools import Client
+        self.logger.debug("get_status_changed_contents, unterminated_job_ids[:10]: %s" % str(unterminated_job_ids[:10]))
 
         full_update_contents = []
         chunksize = 2000
         chunks = [unterminated_job_ids[i:i + chunksize] for i in range(0, len(unterminated_job_ids), chunksize)]
         for chunk in chunks:
-            jobs_list = Client.getJobStatus(chunk, verbose=0)[1]
+            # jobs_list = Client.getJobStatus(chunk, verbose=0)[1]
+            jobs_list = self.get_panda_job_status(chunk)
             for job_info in jobs_list:
                 panda_id = job_info.PandaID
                 map_id = panda_id_to_map_ids[panda_id]
@@ -678,10 +756,10 @@ class DomaPanDAWork(Work):
 
         return update_contents
 
-    def poll_panda_task(self, processing=None, input_output_maps=None):
+    def poll_panda_task_old(self, processing=None, input_output_maps=None):
         task_id = None
         try:
-            from pandatools import Client
+            from pandaclient import Client
 
             jobs_ids = None
             if processing:
@@ -692,8 +770,9 @@ class DomaPanDAWork(Work):
 
                 if task_id:
                     # ret_ids = Client.getPandaIDsWithTaskID(task_id, verbose=False)
+                    self.logger.debug("poll_panda_task, task_id: %s" % str(task_id))
                     task_info = Client.getJediTaskDetails({'jediTaskID': task_id}, True, True, verbose=False)
-                    self.logger.info("poll_panda_task, task_info: %s" % str(task_info))
+                    self.logger.debug("poll_panda_task, task_info[0]: %s" % str(task_info[0]))
                     if task_info[0] != 0:
                         self.logger.warn("poll_panda_task %s, error getting task status, task_info: %s" % (task_id, str(task_info)))
                         return ProcessingStatus.Submitting, {}
@@ -748,10 +827,220 @@ class DomaPanDAWork(Work):
             # raise exceptions.IDDSException(msg)
         return ProcessingStatus.Submitting, []
 
+    def poll_panda_jobs(self, job_ids):
+        job_ids = list(job_ids)
+        self.logger.debug("poll_panda_jobs, poll_panda_jobs_chunk_size: %s, job_ids[:10]: %s" % (self.poll_panda_jobs_chunk_size, str(job_ids[:10])))
+
+        # updated_map_ids = []
+        inputname_jobid_map = {}
+        chunksize = self.poll_panda_jobs_chunk_size
+        chunks = [job_ids[i:i + chunksize] for i in range(0, len(job_ids), chunksize)]
+        for chunk in chunks:
+            # jobs_list = Client.getJobStatus(chunk, verbose=0)[1]
+            jobs_list = self.get_panda_job_status(chunk)
+            if jobs_list:
+                self.logger.debug("poll_panda_jobs, input jobs: %s, output_jobs: %s" % (len(chunk), len(jobs_list)))
+                for job_info in jobs_list:
+                    job_status = self.get_content_status_from_panda_status(job_info)
+                    if job_info and job_info.Files and len(job_info.Files) > 0:
+                        for job_file in job_info.Files:
+                            # if job_file.type in ['log']:
+                            if job_file.type not in ['pseudo_input']:
+                                continue
+                            if ':' in job_file.lfn:
+                                pos = job_file.lfn.find(":")
+                                input_file = job_file.lfn[pos + 1:]
+                                # input_file = job_file.lfn.split(':')[1]
+                            else:
+                                input_file = job_file.lfn
+                            inputname_jobid_map[input_file] = {'panda_id': job_info.PandaID, 'status': job_status}
+            else:
+                self.logger.warn("poll_panda_jobs, input jobs: %s, output_jobs: %s" % (len(chunk), jobs_list))
+        return inputname_jobid_map
+
+    def get_job_maps(self, input_output_maps):
+        inputname_mapid_map = {}
+        finished_jobs, failed_jobs = [], []
+        for map_id in input_output_maps:
+            inputs = input_output_maps[map_id]['inputs']
+            outputs = input_output_maps[map_id]['outputs']
+            outputs_short = []
+            for content in outputs:
+                outputs_short.append({'content_id': content['content_id'],
+                                      'status': content['status'],
+                                      'substatus': content['substatus'],
+                                      'content_metadata': content['content_metadata']})
+
+                if content['status'] in [ContentStatus.Available]:
+                    if 'panda_id' in content['content_metadata']:
+                        finished_jobs.append(content['content_metadata']['panda_id'])
+                elif content['status'] in [ContentStatus.Failed, ContentStatus.FinalFailed,
+                                           ContentStatus.Lost, ContentStatus.Deleted,
+                                           ContentStatus.Missing]:
+                    if 'panda_id' in content['content_metadata']:
+                        failed_jobs.append(content['content_metadata']['panda_id'])
+            for content in inputs:
+                inputname_mapid_map[content['name']] = {'map_id': map_id,
+                                                        'outputs': outputs_short}
+        return finished_jobs + failed_jobs, inputname_mapid_map
+
+    def get_update_contents(self, inputnames, inputname_mapid_map, inputname_jobid_map):
+        self.logger.debug("get_update_contents, inputnames[:5]: %s" % str(inputnames[:5]))
+        self.logger.debug("get_update_contents, inputname_mapid_map[:5]: %s" % str({k: inputname_mapid_map[k] for k in inputnames[:5]}))
+        self.logger.debug("get_update_contents, inputname_jobid_map[:5]: %s" % str({k: inputname_jobid_map[k] for k in inputnames[:5]}))
+
+        update_contents = []
+        num_updated_contents, num_unupdated_contents = 0, 0
+        for inputname in inputnames:
+            panda_id_status = inputname_jobid_map[inputname]
+            panda_id = panda_id_status['panda_id']
+            panda_status = panda_id_status['status']
+            map_id_contents = inputname_mapid_map[inputname]
+            contents = map_id_contents['outputs']
+            for content in contents:
+                if content['substatus'] != panda_status:
+                    if 'panda_id' in content['content_metadata'] and content['content_metadata']['panda_id']:
+                        # if content['content_metadata']['panda_id'] != job_info.PandaID:
+                        if content['content_metadata']['panda_id'] < panda_id:
+                            # new panda id is the bigger one.
+                            if 'old_panda_id' not in content['content_metadata']:
+                                content['content_metadata']['old_panda_id'] = []
+                            if content['content_metadata']['panda_id'] not in content['content_metadata']['old_panda_id']:
+                                content['content_metadata']['old_panda_id'].append(content['content_metadata']['panda_id'])
+                            content['content_metadata']['panda_id'] = panda_id
+                            content['substatus'] = panda_status
+                        elif content['content_metadata']['panda_id'] > panda_id:
+                            if 'old_panda_id' not in content['content_metadata']:
+                                content['content_metadata']['old_panda_id'] = []
+                            if panda_id not in content['content_metadata']['old_panda_id']:
+                                content['content_metadata']['old_panda_id'].append(panda_id)
+                            # content['content_metadata']['panda_id'] = content['content_metadata']['panda_id']
+                            # content['substatus'] = panda_status
+                        else:
+                            pass
+                            # content['content_metadata']['panda_id'] = panda_id
+                            content['substatus'] = panda_status
+                    else:
+                        content['content_metadata']['panda_id'] = panda_id
+                        content['substatus'] = panda_status
+
+                    update_contents.append(content)
+                    num_updated_contents += 1
+                else:
+                    num_unupdated_contents += 1
+        self.logger.debug("get_update_contents, num_updated_contents: %s, num_unupdated_contents: %s" % (num_updated_contents, num_unupdated_contents))
+        self.logger.debug("get_update_contents, update_contents[:5]: %s" % (str(update_contents[:5])))
+        return update_contents
+
+    def poll_panda_task(self, processing=None, input_output_maps=None):
+        task_id = None
+        try:
+            from pandaclient import Client
+
+            if processing:
+                proc = processing['processing_metadata']['processing']
+                task_id = proc.workload_id
+                if task_id is None:
+                    task_id = self.get_panda_task_id(processing)
+
+                if task_id:
+                    # ret_ids = Client.getPandaIDsWithTaskID(task_id, verbose=False)
+                    self.logger.debug("poll_panda_task, task_id: %s" % str(task_id))
+                    task_info = Client.getJediTaskDetails({'jediTaskID': task_id}, True, True, verbose=False)
+                    self.logger.debug("poll_panda_task, task_info[0]: %s" % str(task_info[0]))
+                    if task_info[0] != 0:
+                        self.logger.warn("poll_panda_task %s, error getting task status, task_info: %s" % (task_id, str(task_info)))
+                        return ProcessingStatus.Submitting, []
+
+                    task_info = task_info[1]
+
+                    processing_status = self.get_processing_status_from_panda_status(task_info["status"])
+
+                    if processing_status in [ProcessingStatus.SubFinished]:
+                        if self.retry_number < self.num_retries:
+                            self.reactivate_processing(processing)
+                            processing_status = ProcessingStatus.Submitted
+                            self.retry_number += 1
+                    if processing_status in [ProcessingStatus.Broken]:
+                        self.logger.error("poll_panda_task, task_id: %s is broken. retry_number: %s, num_retries: %s" % (str(task_id), self.retry_number, self.num_retries))
+                        if self.num_retries == 0:
+                            self.num_retries = 1
+                        if self.retry_number < self.num_retries:
+                            self.retry_number += 1
+                            self.logger.error("poll_panda_task, task_id: %s is broken. resubmit the task. retry_number: %s, num_retries: %s" % (str(task_id), self.retry_number, self.num_retries))
+                            self.resubmit_processing(processing)
+                            return ProcessingStatus.Submitting, []
+                    all_jobs_ids = task_info['PandaID']
+
+                    terminated_jobs, inputname_mapid_map = self.get_job_maps(input_output_maps)
+                    self.logger.debug("poll_panda_task, task_id: %s, all jobs: %s, terminated_jobs: %s" % (str(task_id), len(all_jobs_ids), len(terminated_jobs)))
+
+                    all_jobs_ids = set(all_jobs_ids)
+                    terminated_jobs = set(terminated_jobs)
+                    unterminated_jobs = all_jobs_ids - terminated_jobs
+
+                    inputname_jobid_map = self.poll_panda_jobs(unterminated_jobs)
+                    intersection_keys = set(inputname_mapid_map.keys()) & set(inputname_jobid_map.keys())
+
+                    updated_contents = self.get_update_contents(list(intersection_keys), inputname_mapid_map, inputname_jobid_map)
+
+                    final_update_contents = []
+                    if processing_status in [ProcessingStatus.SubFinished, ProcessingStatus.Finished, ProcessingStatus.Failed]:
+                        if updated_contents:
+                            # there are still polling contents, should not terminate the task.
+                            log_warn = "Processing (%s) with panda task id (%s) is %s, however there are still updated_contents[:5]: %s" % (processing['processing_id'],
+                                                                                                                                            task_id,
+                                                                                                                                            processing_status,
+                                                                                                                                            str(updated_contents[:5]))
+                            log_warn = log_warn + ". Keep the processing status as running now."
+                            self.logger.warn(log_warn)
+                            processing_status = ProcessingStatus.Running
+                        elif list(unterminated_jobs):
+                            log_warn = "Processing (%s) with panda task id (%s) is %s, however there are still unterminated_jobs[:5]: %s" % (processing['processing_id'],
+                                                                                                                                             task_id,
+                                                                                                                                             processing_status,
+                                                                                                                                             str(list(unterminated_jobs)[:5]))
+                            log_warn = log_warn + ". Keep the processing status as running now."
+                            self.logger.warn(log_warn)
+                            processing_status = ProcessingStatus.Running
+                        else:
+                            # unsubmitted_inputnames = set(inputname_mapid_map.keys()) - set(inputname_jobid_map.keys())
+                            # unsubmitted_inputnames = list(unsubmitted_inputnames)
+                            # if unsubmitted_inputnames:
+                            #     log_warn = "Processing (%s) with panda task id (%s) is %s, however there are still unsubmitted_inputnames[:5]: %s" % (processing['processing_id'],
+                            #                                                                                                                           task_id,
+                            #                                                                                                                           processing_status,
+                            #                                                                                                                           str(unsubmitted_inputnames[:5]))
+                            #     log_warn = log_warn + ". Keep the processing status as running now."
+                            #     self.logger.warn(log_warn)
+                            #     processing_status = ProcessingStatus.Running
+
+                            for inputname in inputname_mapid_map:
+                                map_id_contents = inputname_mapid_map[inputname]
+                                contents = map_id_contents['outputs']
+                                for content in contents:
+                                    if (content['substatus'] not in [ContentStatus.Available, ContentStatus.FakeAvailable, ContentStatus.FinalFailed]):
+                                        content['content_metadata']['old_final_status'] = content['substatus']
+                                        content['substatus'] = ContentStatus.FinalFailed
+                                        # final_update_contents.append(content)     # TODO: mark other contents to Missing
+
+                            if final_update_contents:
+                                processing_status = ProcessingStatus.Running
+                    return processing_status, updated_contents + final_update_contents
+                else:
+                    return ProcessingStatus.Failed, []
+        except Exception as ex:
+            msg = "Failed to check the processing (%s) status: %s" % (str(processing['processing_id']), str(ex))
+            self.logger.error(msg)
+            self.logger.error(ex)
+            self.logger.error(traceback.format_exc())
+            # raise exceptions.IDDSException(msg)
+        return ProcessingStatus.Submitting, []
+
     def kill_processing(self, processing):
         try:
             if processing:
-                from pandatools import Client
+                from pandaclient import Client
                 proc = processing['processing_metadata']['processing']
                 task_id = proc.workload_id
                 # task_id = processing['processing_metadata']['task_id']
@@ -764,7 +1053,7 @@ class DomaPanDAWork(Work):
     def kill_processing_force(self, processing):
         try:
             if processing:
-                from pandatools import Client
+                from pandaclient import Client
                 proc = processing['processing_metadata']['processing']
                 task_id = proc.workload_id
                 # task_id = processing['processing_metadata']['task_id']
@@ -777,7 +1066,7 @@ class DomaPanDAWork(Work):
     def reactivate_processing(self, processing):
         try:
             if processing:
-                from pandatools import Client
+                from pandaclient import Client
                 # task_id = processing['processing_metadata']['task_id']
                 proc = processing['processing_metadata']['processing']
                 task_id = proc.workload_id
@@ -799,7 +1088,7 @@ class DomaPanDAWork(Work):
         update_processing = {}
         reset_expired_at = False
         reactive_contents = []
-        # self.logger.debug("poll_processing_updates, input_output_maps: %s" % str(input_output_maps))
+        self.logger.debug("poll_processing_updates, input_output_maps.keys[:5]: %s" % str(list(input_output_maps.keys())[:5]))
 
         if processing:
             proc = processing['processing_metadata']['processing']
@@ -811,8 +1100,8 @@ class DomaPanDAWork(Work):
                 proc.polling_retries = 0
             elif proc.tosuspend:
                 self.logger.info("Suspending processing (processing id: %s, jediTaskId: %s)" % (processing['processing_id'], proc.workload_id))
-                self.kill_processing_force(processing)
-                # self.kill_processing(processing)
+                # self.kill_processing_force(processing)
+                self.kill_processing(processing)
                 proc.tosuspend = False
                 proc.polling_retries = 0
             elif proc.toresume:
@@ -841,7 +1130,7 @@ class DomaPanDAWork(Work):
 
             processing_status, poll_updated_contents = self.poll_panda_task(processing=processing, input_output_maps=input_output_maps)
             self.logger.debug("poll_processing_updates, processing_status: %s" % str(processing_status))
-            self.logger.debug("poll_processing_updates, update_contents: %s" % str(poll_updated_contents))
+            self.logger.debug("poll_processing_updates, update_contents[:10]: %s" % str(poll_updated_contents[:10]))
 
             if poll_updated_contents:
                 proc.has_new_updates()
@@ -895,10 +1184,10 @@ class DomaPanDAWork(Work):
 
         self.logger.debug("poll_processing_updates, task: %s, update_processing: %s" %
                           (proc.workload_id, str(update_processing)))
-        self.logger.debug("poll_processing_updates, task: %s, updated_contents: %s" %
-                          (proc.workload_id, str(updated_contents)))
-        self.logger.debug("poll_processing_updates, task: %s, reactive_contents: %s" %
-                          (proc.workload_id, str(reactive_contents)))
+        self.logger.debug("poll_processing_updates, task: %s, updated_contents[:100]: %s" %
+                          (proc.workload_id, str(updated_contents[:100])))
+        self.logger.debug("poll_processing_updates, task: %s, reactive_contents[:100]: %s" %
+                          (proc.workload_id, str(reactive_contents[:100])))
         return update_processing, updated_contents + reactive_contents, {}
 
     def get_status_statistics(self, registered_input_output_maps):
